@@ -39,6 +39,7 @@ VALUE_UNITS = (
     ('Ohms', '\u03A9'),
     ('mOhms', 'm\u03A9'),
     ('kOhms', 'k\u03A9'),
+    ('MOhms', 'M\u03A9'),
     ('F', 'F'),
     ('pF', 'pF'),
     ('nF', 'nF'),
@@ -52,6 +53,7 @@ VALUE_UNITS = (
     ('C', '\u00B0C'),
     ('F', '\u00B0F'),
     ('H', 'H'),
+    ('nH', 'nH'),
     ('mH', 'mH'),
     ('uH', '\u03BCH'),
     ('Hz', 'Hz'),
