@@ -19,3 +19,14 @@ def google_authenticated(function):
     wrap.__doc__ = function.__doc__
     wrap.__name__ = function.__name__
     return wrap
+
+def organization_admin(function):
+    def wrap(request, *args, **kwargs):
+        if request.user.bom_profile().role != 'A':
+            messages.error(request, "You don't have permission to perform this action.")
+            return HttpResponseRedirect(request.META.get('HTTP_REFERER'), reverse('bom:home'))
+        return function(request, *args, **kwargs)
+
+    wrap.__doc__ = function.__doc__
+    wrap.__name__ = function.__name__
+    return wrap
